@@ -1,10 +1,10 @@
 
-#define WANT_DIRFD
+#define EXTUNIX_WANT_DIRFD
 #include "config.h"
 
 #include <errno.h>
 
-#if defined(HAVE_DIRFD)
+#if defined(EXTUNIX_HAVE_DIRFD)
 
 CAMLprim value caml_extunix_dirfd(value v_dir)
 {
